@@ -1,17 +1,17 @@
 import logging
 import os
 import re
-from abc import abstractmethod
+from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from functools import cache
 from io import TextIOWrapper
-from typing import Generator, Optional
+from typing import Generator, Optional, cast
 
 from pygments.lexer import Lexer
-from pygments.lexers import guess_lexer_for_filename
+from pygments.lexers import get_lexer_for_filename
 from pygments.util import ClassNotFound
 from tree_sitter import Node, Point
-from tree_sitter_language_pack import get_parser
+from tree_sitter_language_pack import SupportedLanguage, get_parser
 
 from vectorcode.cli_utils import Config
 
@@ -31,8 +31,23 @@ class Chunk:
     def __str__(self):
         return self.text
 
+    def export_dict(self):
+        if self.start is not None:
+            return {
+                "text": self.text,
+                "start": {"row": self.start.row, "column": self.start.column},
+                "end": {"row": self.end.row, "column": self.end.column},
+            }
+        else:
+            return {"text": self.text}
 
-class ChunkerBase:
+
+@dataclass
+class ChunkOpts:
+    start_pos: Point
+
+
+class ChunkerBase(ABC):  # pragma: nocover
     def __init__(self, config: Optional[Config] = None) -> None:
         if config is None:
             config = Config()
@@ -42,7 +57,9 @@ class ChunkerBase:
         self.config = config
 
     @abstractmethod
-    def chunk(self, data) -> Generator[Chunk, None, None]:
+    def chunk(
+        self, data, opts: Optional[ChunkOpts] = None
+    ) -> Generator[Chunk, None, None]:
         raise NotImplementedError
 
 
@@ -52,14 +69,21 @@ class StringChunker(ChunkerBase):
             config = Config()
         super().__init__(config)
 
-    def chunk(self, data: str):
+    def chunk(self, data: str, opts: Optional[ChunkOpts] = None):
+        start_pos = Point(row=1, column=0)
+        if opts is not None:
+            start_pos = opts.start_pos
+
         logger.info("Started chunking with StringChunker.")
         logger.debug(f"{data=}")
         if self.config.chunk_size < 0:
             yield Chunk(
                 text=data,
-                start=Point(row=1, column=0),
-                end=Point(row=1, column=len(data)),
+                start=start_pos,
+                end=Point(
+                    row=data.count("\n") + start_pos.row,
+                    column=len(data.split("\n")[-1]) - 1,
+                ),
             )
         else:
             step_size = max(
@@ -68,10 +92,26 @@ class StringChunker(ChunkerBase):
             i = 0
             while i < len(data):
                 chunk_text = data[i : i + self.config.chunk_size]
+
+                start_lines_before_chunk = data[:i].count("\n")
+                chunk_start_row = start_pos.row + start_lines_before_chunk
+                if start_lines_before_chunk == 0:
+                    chunk_start_column = start_pos.column + i
+                else:
+                    last_newline_idx_before_i = data.rfind("\n", 0, i)
+                    chunk_start_column = i - (last_newline_idx_before_i + 1)
+
+                chunk_end_row = chunk_start_row + chunk_text.count("\n")
+
+                if "\n" in chunk_text:
+                    chunk_end_column = len(chunk_text.split("\n")[-1]) - 1
+                else:
+                    chunk_end_column = chunk_start_column + len(chunk_text) - 1
+
                 yield Chunk(
                     text=chunk_text,
-                    start=Point(row=1, column=i),
-                    end=Point(row=1, column=i + len(chunk_text) - 1),
+                    start=Point(row=chunk_start_row, column=chunk_start_column),
+                    end=Point(row=chunk_end_row, column=chunk_end_column),
                 )
                 if i + self.config.chunk_size >= len(data):
                     break
@@ -84,8 +124,10 @@ class FileChunker(ChunkerBase):
             config = Config()
         super().__init__(config)
 
-    def chunk(self, data: TextIOWrapper) -> Generator[Chunk, None, None]:
-        logger.info("Started chunking using FileChunker.", data.name)
+    def chunk(
+        self, data: TextIOWrapper, opts: Optional[ChunkOpts] = None
+    ) -> Generator[Chunk, None, None]:
+        logger.info("Started chunking %s using FileChunker.", data.name)
         lines = data.readlines()
         if len(lines) == 0:
             return
@@ -141,6 +183,7 @@ class TreeSitterChunker(ChunkerBase):
         if config is None:
             config = Config()
         super().__init__(config)
+        self._fallback_chunker = StringChunker(config)
 
     def __chunk_node(
         self, node: Node, text_bytes: bytes
@@ -153,6 +196,14 @@ class TreeSitterChunker(ChunkerBase):
         prev_node = None
         current_start = None
 
+        logger.debug("nbr children: %s", len(node.children))
+        # if node has no children we fallback to the string chunker
+        if len(node.children) == 0 and node.text:
+            logger.debug("No children, falling back to string chunker")
+            yield from self._fallback_chunker.chunk(
+                node.text.decode(), ChunkOpts(start_pos=node.start_point)
+            )
+
         for child in node.children:
             child_bytes = text_bytes[child.start_byte : child.end_byte]
             child_text = child_bytes.decode()
@@ -233,7 +284,7 @@ class TreeSitterChunker(ChunkerBase):
     @cache
     def __guess_type(self, path: str, content: str) -> Optional[Lexer]:
         try:
-            return guess_lexer_for_filename(path, content)
+            return get_lexer_for_filename(path, content)
 
         except ClassNotFound:
             return None
@@ -272,7 +323,50 @@ class TreeSitterChunker(ChunkerBase):
             lines = fin.readlines()
         return lines
 
-    def chunk(self, data: str) -> Generator[Chunk, None, None]:
+    def __get_parser_from_config(self, file_path: str):
+        """
+        Get parser based on filetype_map config.
+        """
+        filetype_map = self.config.filetype_map
+        if not filetype_map:
+            logger.debug("filetype_map is empty in config.")
+            return None
+
+        filename = os.path.basename(file_path)
+        extension = os.path.splitext(file_path)[1]
+        if extension.startswith("."):
+            extension = extension[1:]
+        logger.debug(f"Checking filetype map for extension '{extension}' in {filename}")
+        for _language, patterns in filetype_map.items():
+            language = _language.lower()
+            for pattern in patterns:
+                try:
+                    if re.search(pattern, extension):
+                        logger.debug(
+                            f"'{filename}' extension matches pattern '{pattern}' for language '{language}'. Attempting to load parser."
+                        )
+                        parser = get_parser(cast(SupportedLanguage, language))
+                        logger.debug(
+                            f"Found parser for language '{language}' from config."
+                        )
+                        return parser
+                except re.error as e:
+                    e.add_note(
+                        f"\nInvalid regex pattern '{pattern}' for language '{language}' in filetype_map"
+                    )
+                    raise
+                except LookupError as e:
+                    e.add_note(
+                        f"\nTreeSitter Parser for language '{language}' not found. Please check your filetype_map config."
+                    )
+                    raise
+
+        logger.debug(f"No matching filetype map entry found for {filename}.")
+        return None
+
+    def chunk(
+        self, data: str, opts: Optional[ChunkOpts] = None
+    ) -> Generator[Chunk, None, None]:
         """
         data: path to the file
         """
@@ -287,27 +381,30 @@ class TreeSitterChunker(ChunkerBase):
             return
         parser = None
         language = None
-        lexer = self.__guess_type(data, content)
-        if lexer is not None:
-            lang_names = [lexer.name]
-            lang_names.extend(lexer.aliases)
-            for name in lang_names:
-                try:
-                    parser = get_parser(name.lower())
-                    if parser is not None:
-                        language = name.lower()
-                        logger.debug(
-                            "Detected %s filetype for treesitter chunking.", language
-                        )
-                        break
-                except LookupError:  # pragma: nocover
-                    pass
+        parser = self.__get_parser_from_config(data)
+        if parser is None:
+            lexer = self.__guess_type(data, content)
+            if lexer is not None:
+                lang_names = [lexer.name]
+                lang_names.extend(lexer.aliases)
+                for name in lang_names:
+                    try:
+                        parser = get_parser(cast(SupportedLanguage, name.lower()))
+                        if parser is not None:
+                            language = name.lower()
+                            logger.debug(
+                                "Detected %s filetype for treesitter chunking.",
+                                language,
+                            )
+                            break
+                    except LookupError:  # pragma: nocover
+                        pass
 
         if parser is None:
             logger.debug(
                 "Unable to pick a suitable parser. Fall back to naive chunking"
             )
-            yield from StringChunker(self.config).chunk(content)
+            yield from self._fallback_chunker.chunk(content, opts)
         else:
             pattern_str = self.__build_pattern(language=language)
             content_bytes = content.encode()
