@@ -1,5 +1,7 @@
+import os
+import tempfile
 from argparse import ArgumentParser
-from unittest.mock import AsyncMock, MagicMock, patch
+from unittest.mock import AsyncMock, MagicMock, mock_open, patch
 
 import pytest
 from mcp import McpError
@@ -11,6 +13,7 @@ from vectorcode.mcp_main import (
     mcp_server,
     parse_cli_args,
     query_tool,
+    vectorise_files,
 )
 
 
@@ -141,8 +144,8 @@ async def test_query_tool_collection_access_failure():
 
         assert exc_info.value.error.code == 1
         assert (
-            exc_info.value.error.message
-            == "Failed to access the collection at /valid/path. Use `list_collections` tool to get a list of valid paths for this field."
+            "Failed to access the collection at /valid/path. Use `list_collections` tool to get a list of valid paths for this field."
+            in exc_info.value.error.message
         )
 
 
@@ -168,6 +171,125 @@ async def test_query_tool_no_collection():
         )
 
 
+@pytest.mark.asyncio
+async def test_vectorise_tool_invalid_project_root():
+    with (
+        patch("os.path.isdir", return_value=False),
+    ):
+        with pytest.raises(McpError):
+            await vectorise_files(paths=["foo.bar"], project_root=".")
+
+
+@pytest.mark.asyncio
+async def test_vectorise_files_success():
+    with tempfile.TemporaryDirectory() as temp_dir:
+        file_path = f"{temp_dir}/test_file.py"
+        with open(file_path, "w") as f:
+            f.write("def func(): pass")
+
+        with (
+            patch("os.path.isdir", return_value=True),
+            patch("vectorcode.mcp_main.get_project_config") as mock_get_project_config,
+            patch("vectorcode.mcp_main.get_client") as mock_get_client,
+            patch("vectorcode.mcp_main.get_collection") as mock_get_collection,
+            patch("vectorcode.subcommands.vectorise.chunked_add"),
+            patch(
+                "vectorcode.subcommands.vectorise.hash_file", return_value="test_hash"
+            ),
+        ):
+            mock_config = Config(project_root=temp_dir)
+            mock_get_project_config.return_value = mock_config
+            mock_client = AsyncMock()
+            mock_get_client.return_value = mock_client
+            mock_collection = AsyncMock()
+            mock_collection.get.return_value = {"ids": [], "metadatas": []}
+            mock_get_collection.return_value = mock_collection
+            mock_client.get_max_batch_size.return_value = 100
+
+            result = await vectorise_files(paths=[file_path], project_root=temp_dir)
+
+            assert result["add"] == 1
+            mock_get_project_config.assert_called_once_with(temp_dir)
+            mock_get_client.assert_called_once_with(mock_config)
+            mock_get_collection.assert_called_once_with(mock_client, mock_config, True)
+
+
+@pytest.mark.asyncio
+async def test_vectorise_files_collection_access_failure():
+    with (
+        patch("os.path.isdir", return_value=True),
+        patch("vectorcode.mcp_main.get_project_config"),
+        patch("vectorcode.mcp_main.get_client", side_effect=Exception("Client error")),
+        patch("vectorcode.mcp_main.get_collection"),
+    ):
+        with pytest.raises(McpError) as exc_info:
+            await vectorise_files(paths=["file.py"], project_root="/valid/path")
+
+        assert exc_info.value.error.code == 1
+        assert (
+            "Failed to create the collection at /valid/path"
+            in exc_info.value.error.message
+        )
+
+
+@pytest.mark.asyncio
+async def test_vectorise_files_with_exclude_spec():
+    with tempfile.TemporaryDirectory() as temp_dir:
+        file1 = f"{temp_dir}/file1.py"
+        excluded_file = f"{temp_dir}/excluded.py"
+        exclude_spec_file = f"{temp_dir}/.vectorcode/vectorcode.exclude"
+
+        os.makedirs(f"{temp_dir}/.vectorcode")
+        with open(file1, "w") as f:
+            f.write("content1")
+        with open(excluded_file, "w") as f:
+            f.write("content_excluded")
+
+        # Create mock file handles for specific file contents
+        mock_exclude_file_handle = mock_open(read_data="excluded.py").return_value
+
+        def mock_open_side_effect(filename, *args, **kwargs):
+            if filename == exclude_spec_file:
+                return mock_exclude_file_handle
+            # For other files that might be opened, return a generic mock
+            return MagicMock()
+
+        with (
+            patch("os.path.isdir", return_value=True),
+            patch("vectorcode.mcp_main.get_project_config") as mock_get_project_config,
+            patch("vectorcode.mcp_main.get_client") as mock_get_client,
+            patch("vectorcode.mcp_main.get_collection") as mock_get_collection,
+            patch("vectorcode.subcommands.vectorise.chunked_add") as mock_chunked_add,
+            patch(
+                "vectorcode.subcommands.vectorise.hash_file", return_value="test_hash"
+            ),
+            # Patch builtins.open with the custom side effect
+            patch("builtins.open", side_effect=mock_open_side_effect),
+            # Patch os.path.isfile to control which files "exist"
+            patch(
+                "os.path.isfile",
+                side_effect=lambda x: x in [file1, excluded_file, exclude_spec_file],
+            ),
+        ):
+            mock_config = Config(project_root=temp_dir)
+            mock_get_project_config.return_value = mock_config
+            mock_client = AsyncMock()
+            mock_get_client.return_value = mock_client
+            mock_collection = AsyncMock()
+            mock_collection.get.return_value = {"ids": [], "metadatas": []}
+            mock_get_collection.return_value = mock_collection
+            mock_client.get_max_batch_size.return_value = 100
+
+            result = await vectorise_files(
+                paths=[file1, excluded_file], project_root=temp_dir
+            )
+
+            assert result["add"] == 0
+            assert mock_chunked_add.call_count == 0
+            call_args = [call[0][0] for call in mock_chunked_add.call_args_list]
+            assert excluded_file not in call_args
+
+
 @pytest.mark.asyncio
 async def test_mcp_server():
     with (
@@ -188,7 +310,7 @@ async def test_mcp_server():
 
         await mcp_server()
 
-        assert mock_add_tool.call_count == 2
+        assert mock_add_tool.call_count == 3
 
 
 @pytest.mark.asyncio
@@ -223,7 +345,7 @@ async def test_mcp_server_ls_on_start():
 
         await mcp_server()
 
-        assert mock_add_tool.call_count == 2
+        assert mock_add_tool.call_count == 3
         mock_get_collections.assert_called()
 
 
